"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.sizeSnapshot = void 0;

var _path = require("path");

var _terser = require("terser");

var _gzipSize = _interopRequireDefault(require("gzip-size"));

var _bytes = _interopRequireDefault(require("bytes"));

var _chalk = _interopRequireDefault(require("chalk"));

var _treeshakeWithRollup = require("./treeshakeWithRollup.js");

var _treeshakeWithWebpack = require("./treeshakeWithWebpack.js");

var snapshot = _interopRequireWildcard(require("./snapshot.js"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const validateOptions = options => {
  const optionsKeys = ["snapshotPath", "matchSnapshot", "threshold", "printInfo"];
  const invalidKeys = Object.keys(options).filter(d => !optionsKeys.includes(d));

  const wrap = d => `"${d}"`;

  if (1 === invalidKeys.length) {
    throw Error(`Option ${wrap(invalidKeys[0])} is invalid`);
  }

  if (1 < invalidKeys.length) {
    throw Error(`Options ${invalidKeys.map(wrap).join(", ")} are invalid`);
  }
};

const bytesConfig = {
  thousandsSeparator: ",",
  unitSeparator: " ",
  unit: "B"
};

const formatSize = d => _chalk.default.bold(_bytes.default.format(d, bytesConfig));

const sizeSnapshot = (options = {}) => {
  validateOptions(options);
  const snapshotPath = options.snapshotPath || (0, _path.join)(process.cwd(), ".size-snapshot.json");
  const shouldMatchSnapshot = options.matchSnapshot === true;
  const shouldPrintInfo = options.printInfo !== false;
  const threshold = options.threshold == null ? 0 : options.threshold;
  return {
    name: "size-snapshot",

    transformBundle(source, outputOptions) {
      const format = outputOptions.format;
      const output = outputOptions.file;
      const shouldTreeshake = format === "es";

      if (typeof output !== "string") {
        throw Error("output file in rollup options should be specified");
      }

      const minified = (0, _terser.minify)(source).code;

      const treeshakeSize = code => Promise.all([(0, _treeshakeWithRollup.treeshakeWithRollup)(code), (0, _treeshakeWithWebpack.treeshakeWithWebpack)(code)]);

      return Promise.all([(0, _gzipSize.default)(minified), shouldTreeshake ? treeshakeSize(source) : [{
        code: 0,
        import_statements: 0
      }, {
        code: 0
      }]]).then(([gzippedSize, [rollupSize, webpackSize]]) => {
        const sizes = {
          bundled: source.length,
          minified: minified.length,
          gzipped: gzippedSize
        };
        let infoString = "\n" + `Computed sizes of "${output}" with "${format === "es" ? "esm" : format}" format\n` + `  bundled: ${formatSize(sizes.bundled)}\n` + `  minified with terser: ${formatSize(sizes.minified)}\n` + `  minified and gzipped: ${formatSize(sizes.gzipped)}\n`;

        const formatMsg = (msg, size) => {
          return `  ${msg}: ${formatSize(size)}\n`;
        };

        if (shouldTreeshake) {
          sizes.treeshaked = {
            rollup: rollupSize,
            webpack: webpackSize
          };
          infoString += formatMsg("treeshaked with rollup and minified", rollupSize.code);
          infoString += formatMsg("  import statements size", rollupSize.import_statements);
          infoString += formatMsg("treeshaked with webpack in production mode", webpackSize.code);
        }

        const snapshotParams = {
          snapshotPath,
          name: output,
          data: sizes,
          threshold
        };

        if (shouldMatchSnapshot) {
          snapshot.match(snapshotParams);
        } else {
          if (shouldPrintInfo) {
            console.info(infoString);
          }

          snapshot.write(snapshotParams);
        }

        return null;
      });
    }

  };
};

exports.sizeSnapshot = sizeSnapshot;